@@ -49,6 +49,15 @@ NVML_COMPUTEMODE_EXCLUSIVE_THREAD  = 1
 NVML_COMPUTEMODE_PROHIBITED        = 2
 NVML_COMPUTEMODE_EXCLUSIVE_PROCESS = 3
 
+_nvmlMemoryLocation_t = c_uint
+NVML_MEMORY_LOCATION_L1_CACHE = 0
+NVML_MEMORY_LOCATION_L2_CACHE = 1
+NVML_MEMORY_LOCATION_DEVICE_MEMORY = 2
+NVML_MEMORY_LOCATION_REGISTER_FILE = 3
+NVML_MEMORY_LOCATION_TEXTURE_MEMORY = 4
+NVML_MEMORY_LOCATION_COUNT = 5
+
+# These are deprecated, instead use _nvmlMemoryErrorType_t
 _nvmlEccBitType_t = c_uint
 NVML_SINGLE_BIT_ECC    = 0
 NVML_DOUBLE_BIT_ECC    = 1
@@ -57,10 +66,16 @@ _nvmlEccCounterType_t = c_uint
 NVML_VOLATILE_ECC      = 0
 NVML_AGGREGATE_ECC     = 1
 
+_nvmlMemoryErrorType_t = c_uint
+NVML_MEMORY_ERROR_TYPE_CORRECTED   = 0
+NVML_MEMORY_ERROR_TYPE_UNCORRECTED = 1
+NVML_MEMORY_ERROR_TYPE_COUNT       = 2
+
 _nvmlClockType_t = c_uint
 NVML_CLOCK_GRAPHICS  = 0
 NVML_CLOCK_SM        = 1
 NVML_CLOCK_MEM       = 2
+NVML_CLOCK_COUNT     = 3
 
 _nvmlDriverModel_t = c_uint
 NVML_DRIVER_WDDM       = 0
@@ -89,6 +104,7 @@ _nvmlInforomObject_t = c_uint
 NVML_INFOROM_OEM            = 0
 NVML_INFOROM_ECC            = 1
 NVML_INFOROM_POWER          = 2
+NVML_INFOROM_COUNT          = 3
 
 _nvmlReturn_t = c_uint
 NVML_SUCCESS                   = 0
@@ -101,7 +117,11 @@ NVML_ERROR_NOT_FOUND           = 6
 NVML_ERROR_INSUFFICIENT_SIZE   = 7
 NVML_ERROR_INSUFFICIENT_POWER  = 8
 NVML_ERROR_DRIVER_NOT_LOADED   = 9
-NVML_ERROR_TIMEOUT             = 10,
+NVML_ERROR_TIMEOUT             = 10
+NVML_ERROR_IRQ_ISSUE           = 11
+NVML_ERROR_LIBRARY_NOT_FOUND   = 12
+NVML_ERROR_FUNCTION_NOT_FOUND  = 13
+NVML_ERROR_CORRUPTED_INFOROM   = 14
 NVML_ERROR_UNKNOWN             = 999
 
 _nvmlFanState_t = c_uint
@@ -111,6 +131,11 @@ NVML_FAN_FAILED             = 1
 _nvmlLedColor_t = c_uint
 NVML_LED_COLOR_GREEN        = 0
 NVML_LED_COLOR_AMBER        = 1
+     
+_nvmlGpuOperationMode_t = c_uint
+NVML_GOM_ALL_ON                 = 0 
+NVML_GOM_COMPUTE                = 1
+NVML_GOM_LOW_DP                 = 2
 
 # C preprocessor defined values
 nvmlFlagDefault             = 0
@@ -300,17 +325,35 @@ c_nvmlEventSet_t = POINTER(struct_c_nvmlEventSet_t)
 
 nvmlEventTypeSingleBitEccError     = 0x0000000000000001
 nvmlEventTypeDoubleBitEccError     = 0x0000000000000002
-nvmlEventTypePState                = 0x0000000000000004     
-nvmlEventTypeXidCriticalError      = 0x0000000000000008     
-nvmlEventTypeNone                  = 0x0000000000000000  
+nvmlEventTypePState                = 0x0000000000000004
+nvmlEventTypeXidCriticalError      = 0x0000000000000008
+nvmlEventTypeClock                 = 0x0000000000000010
+nvmlEventTypeNone                  = 0x0000000000000000
 nvmlEventTypeAll                   = (
                                         nvmlEventTypeNone |
                                         nvmlEventTypeSingleBitEccError |
                                         nvmlEventTypeDoubleBitEccError |
                                         nvmlEventTypePState |
+                                        nvmlEventTypeClock |
                                         nvmlEventTypeXidCriticalError
                                      )
 
+## Clock Throttle Reasons defines
+nvmlClocksThrottleReasonGpuIdle           = 0x0000000000000001
+nvmlClocksThrottleReasonUserDefinedClocks = 0x0000000000000002
+nvmlClocksThrottleReasonSwPowerCap        = 0x0000000000000004
+nvmlClocksThrottleReasonHwSlowdown        = 0x0000000000000008
+nvmlClocksThrottleReasonUnknown           = 0x8000000000000000
+nvmlClocksThrottleReasonNone              = 0x0000000000000000
+nvmlClocksThrottleReasonAll               = (
+                                               nvmlClocksThrottleReasonNone |
+                                               nvmlClocksThrottleReasonGpuIdle |
+                                               nvmlClocksThrottleReasonUserDefinedClocks |
+                                               nvmlClocksThrottleReasonSwPowerCap |
+                                               nvmlClocksThrottleReasonHwSlowdown |
+                                               nvmlClocksThrottleReasonUnknown
+                                            ) 
+
 class c_nvmlEventData_t(Structure):
     _fields_ = [
         ('device', c_nvmlDevice_t),
@@ -560,7 +603,30 @@ def nvmlDeviceGetInforomVersion(handle, infoRomObject):
 	         c_version, c_uint(NVML_DEVICE_INFOROM_VERSION_BUFFER_SIZE))
     _nvmlCheckReturn(ret)
     return c_version.value
-    
+
+# Added in 4.304
+def nvmlDeviceGetInforomImageVersion(handle):
+    c_version = create_string_buffer(NVML_DEVICE_INFOROM_VERSION_BUFFER_SIZE)
+    fn = _nvmlGetFunctionPointer("nvmlDeviceGetInforomImageVersion")
+    ret = fn(handle, c_version, c_uint(NVML_DEVICE_INFOROM_VERSION_BUFFER_SIZE))
+    _nvmlCheckReturn(ret)
+    return c_version.value
+
+# Added in 4.304
+def nvmlDeviceGetInforomConfigurationChecksum(handle):
+    c_checksum = c_uint()
+    fn = _nvmlGetFunctionPointer("nvmlDeviceGetInforomConfigurationChecksum")
+    ret = fn(handle, byref(c_checksum))
+    _nvmlCheckReturn(ret)
+    return c_checksum.value
+
+# Added in 4.304
+def nvmlDeviceValidateInforom(handle):
+    fn = _nvmlGetFunctionPointer("nvmlDeviceValidateInforom")
+    ret = fn(handle)
+    _nvmlCheckReturn(ret)
+    return None 
+
 def nvmlDeviceGetDisplayMode(handle):
     c_mode = _nvmlEnableState_t()
     fn = _nvmlGetFunctionPointer("nvmlDeviceGetDisplayMode")
@@ -597,6 +663,70 @@ def nvmlDeviceGetMaxClockInfo(handle, type):
     _nvmlCheckReturn(ret)
     return c_clock.value
 
+# Added in 4.304
+def nvmlDeviceGetApplicationsClock(handle, type):
+    c_clock = c_uint()
+    fn = _nvmlGetFunctionPointer("nvmlDeviceGetApplicationsClock")
+    ret = fn(handle, _nvmlClockType_t(type), byref(c_clock))
+    _nvmlCheckReturn(ret)
+    return c_clock.value
+
+# Added in 4.304
+def nvmlDeviceGetSupportedMemoryClocks(handle):
+    # first call to get the size
+    c_count = c_uint(0)
+    fn = _nvmlGetFunctionPointer("nvmlDeviceGetSupportedMemoryClocks")
+    ret = fn(handle, byref(c_count), None)
+    
+    if (ret == NVML_SUCCESS):
+        # special case, no clocks
+        return []
+    elif (ret == NVML_ERROR_INSUFFICIENT_SIZE):
+        # typical case
+        clocks_array = c_uint * c_count.value
+        c_clocks = clocks_array()
+        
+        # make the call again
+        ret = fn(handle, byref(c_count), c_clocks)
+        _nvmlCheckReturn(ret)
+        
+        procs = []
+        for i in range(c_count.value):
+            procs.append(c_clocks[i])
+
+        return procs
+    else:
+        # error case
+        raise NVMLError(ret)
+
+# Added in 4.304
+def nvmlDeviceGetSupportedGraphicsClocks(handle, memoryClockMHz):
+    # first call to get the size
+    c_count = c_uint(0)
+    fn = _nvmlGetFunctionPointer("nvmlDeviceGetSupportedGraphicsClocks")
+    ret = fn(handle, c_uint(memoryClockMHz), byref(c_count), None)
+    
+    if (ret == NVML_SUCCESS):
+        # special case, no clocks
+        return []
+    elif (ret == NVML_ERROR_INSUFFICIENT_SIZE):
+        # typical case
+        clocks_array = c_uint * c_count.value
+        c_clocks = clocks_array()
+        
+        # make the call again
+        ret = fn(handle, c_uint(memoryClockMHz), byref(c_count), c_clocks)
+        _nvmlCheckReturn(ret)
+        
+        procs = []
+        for i in range(c_count.value):
+            procs.append(c_clocks[i])
+
+        return procs
+    else:
+        # error case
+        raise NVMLError(ret)
+
 def nvmlDeviceGetFanSpeed(handle):
     c_speed = c_uint()
     fn = _nvmlGetFunctionPointer("nvmlDeviceGetFanSpeed")
@@ -639,6 +769,23 @@ def nvmlDeviceGetPowerManagementLimit(handle):
     ret = fn(handle, byref(c_limit))
     _nvmlCheckReturn(ret)
     return c_limit.value
+
+# Added in 4.304
+def nvmlDeviceGetPowerManagementLimitConstraints(handle):
+    c_minLimit = c_uint()
+    c_maxLimit = c_uint()
+    fn = _nvmlGetFunctionPointer("nvmlDeviceGetPowerManagementLimitConstraints")
+    ret = fn(handle, byref(c_minLimit), byref(c_maxLimit))
+    _nvmlCheckReturn(ret)
+    return [c_minLimit.value, c_maxLimit.value]
+
+# Added in 4.304
+def nvmlDeviceGetPowerManagementDefaultLimit(handle):
+    c_limit = c_uint()
+    fn = _nvmlGetFunctionPointer("nvmlDeviceGetPowerManagementDefaultLimit")
+    ret = fn(handle, byref(c_limit))
+    _nvmlCheckReturn(ret)
+    return c_limit.value
     
 def nvmlDeviceGetPowerUsage(handle):
     c_watts = c_uint()
@@ -646,6 +793,23 @@ def nvmlDeviceGetPowerUsage(handle):
     ret = fn(handle, byref(c_watts))
     _nvmlCheckReturn(ret)
     return c_watts.value
+
+# Added in 4.304
+def nvmlDeviceGetGpuOperationMode(handle):
+    c_currState = _nvmlGpuOperationMode_t()
+    c_pendingState = _nvmlGpuOperationMode_t()
+    fn = _nvmlGetFunctionPointer("nvmlDeviceGetGpuOperationMode")
+    ret = fn(handle, byref(c_currState), byref(c_pendingState))
+    _nvmlCheckReturn(ret)
+    return [c_currState.value, c_pendingState.value]
+
+# Added in 4.304
+def nvmlDeviceGetCurrentGpuOperationMode(handle):
+    return nvmlDeviceGetGpuOperationMode(handle)[0]
+
+# Added in 4.304
+def nvmlDeviceGetPendingGpuOperationMode(handle):
+    return nvmlDeviceGetGpuOperationMode(handle)[1]
     
 def nvmlDeviceGetMemoryInfo(handle):
     c_memory = c_nvmlMemory_t()
@@ -677,21 +841,34 @@ def nvmlDeviceGetCurrentEccMode(handle):
 def nvmlDeviceGetPendingEccMode(handle):
     return nvmlDeviceGetEccMode(handle)[1]
 
-def nvmlDeviceGetTotalEccErrors(handle, bitType, counterType):
+def nvmlDeviceGetTotalEccErrors(handle, errorType, counterType):
     c_count = c_ulonglong()
     fn = _nvmlGetFunctionPointer("nvmlDeviceGetTotalEccErrors")
-    ret = fn(handle, _nvmlEccBitType_t(bitType),
+    ret = fn(handle, _nvmlMemoryErrorType_t(errorType),
 	         _nvmlEccCounterType_t(counterType), byref(c_count))
     _nvmlCheckReturn(ret)
     return c_count.value
 
-def nvmlDeviceGetDetailedEccErrors(handle, bitType, counterType):
-    c_count = c_nvmlEccErrorCounts_t()
+# This is deprecated, instead use nvmlDeviceGetMemoryErrorCounter
+def nvmlDeviceGetDetailedEccErrors(handle, errorType, counterType):
+    c_counts = c_nvmlEccErrorCounts_t()
     fn = _nvmlGetFunctionPointer("nvmlDeviceGetDetailedEccErrors")
-    ret = fn(handle, _nvmlEccBitType_t(bitType),
-	         _nvmlEccCounterType_t(counterType), byref(c_count))
+    ret = fn(handle, _nvmlMemoryErrorType_t(errorType),
+	         _nvmlEccCounterType_t(counterType), byref(c_counts))
     _nvmlCheckReturn(ret)
-    return c_count
+    return c_counts
+    
+# Added in 4.304
+def nvmlDeviceGetMemoryErrorCounter(handle, errorType, counterType, locationType):
+    c_count = c_ulonglong()
+    fn = _nvmlGetFunctionPointer("nvmlDeviceGetMemoryErrorCounter")
+    ret = fn(handle,
+             _nvmlMemoryErrorType_t(errorType),
+             _nvmlEccCounterType_t(counterType),
+             _nvmlMemoryLocation_t(locationType),
+             byref(c_count))
+    _nvmlCheckReturn(ret)
+    return c_count.value
     
 def nvmlDeviceGetUtilizationRates(handle):
     c_util = c_nvmlUtilization_t()
@@ -796,6 +973,34 @@ def nvmlDeviceSetDriverModel(handle, model):
     _nvmlCheckReturn(ret)
     return None
 
+# Added in 4.304
+def nvmlDeviceSetApplicationsClocks(handle, maxMemClockMHz, maxGraphicsClockMHz):
+    fn = _nvmlGetFunctionPointer("nvmlDeviceSetApplicationsClocks")
+    ret = fn(handle, c_uint(maxMemClockMHz), c_uint(maxGraphicsClockMHz))
+    _nvmlCheckReturn(ret)
+    return None
+    
+# Added in 4.304
+def nvmlDeviceResetApplicationsClocks(handle):
+    fn = _nvmlGetFunctionPointer("nvmlDeviceResetApplicationsClocks")
+    ret = fn(handle)
+    _nvmlCheckReturn(ret)
+    return None
+
+# Added in 4.304
+def nvmlDeviceSetPowerManagementLimit(handle, limit):
+    fn = _nvmlGetFunctionPointer("nvmlDeviceSetPowerManagementLimit")
+    ret = fn(handle, c_uint(limit))
+    _nvmlCheckReturn(ret)
+    return None
+    
+# Added in 4.304
+def nvmlDeviceSetGpuOperationMode(handle, mode):
+    fn = _nvmlGetFunctionPointer("nvmlDeviceSetGpuOperationMode")
+    ret = fn(handle, _nvmlGpuOperationMode_t(mode))
+    _nvmlCheckReturn(ret)
+    return None
+
 # Added in 2.285
 def nvmlEventSetCreate():
     fn = _nvmlGetFunctionPointer("nvmlEventSetCreate")
@@ -835,30 +1040,6 @@ def nvmlEventSetFree(eventSet):
     _nvmlCheckReturn(ret)
     return None
 
-# Added in 2.285
-def nvmlEventDataGetPerformanceState(data):
-    fn = _nvmlGetFunctionPointer("nvmlEventDataGetPerformanceState")
-    pstate = _nvmlPstates_t()
-    ret = fn(byref(data), byref(pstate))
-    _nvmlCheckReturn(ret)
-    return pstate.value
-
-# Added in 2.285
-def nvmlEventDataGetXidCriticalError(data):
-    fn = _nvmlGetFunctionPointer("nvmlEventDataGetXidCriticalError")
-    xid = c_uint()
-    ret = fn(byref(data), byref(xid))
-    _nvmlCheckReturn(ret)
-    return xid.value
-
-# Added in 2.285
-def nvmlEventDataGetEccErrorCount(data):
-    fn = _nvmlGetFunctionPointer("nvmlEventDataGetEccErrorCount")
-    ecc = c_ulonglong()
-    ret = fn(byref(data), byref(ecc))
-    _nvmlCheckReturn(ret)
-    return ecc.value
-
 # Added in 3.295
 def nvmlDeviceOnSameBoard(handle1, handle2):
     fn = _nvmlGetFunctionPointer("nvmlDeviceOnSameBoard")
@@ -899,5 +1080,18 @@ def nvmlDeviceGetMaxPcieLinkWidth(handle):
     _nvmlCheckReturn(ret)
     return width.value
 
+# Added in 4.304
+def nvmlDeviceGetSupportedClocksThrottleReasons(handle):
+    c_reasons= c_ulonglong()
+    fn = _nvmlGetFunctionPointer("nvmlDeviceGetSupportedClocksThrottleReasons")
+    ret = fn(handle, byref(c_reasons))
+    _nvmlCheckReturn(ret)
+    return c_reasons.value
 
-
+# Added in 4.304
+def nvmlDeviceGetCurrentClocksThrottleReasons(handle):
+    c_reasons= c_ulonglong()
+    fn = _nvmlGetFunctionPointer("nvmlDeviceGetCurrentClocksThrottleReasons")
+    ret = fn(handle, byref(c_reasons))
+    _nvmlCheckReturn(ret)
+    return c_reasons.value
